# 接入API

module EasyWeixin
	class Api
		include Menu

		def initialize(app_id, app_secret)
			@app_id = app_id
			@app_secret = app_secret
			@redis_key = "weixin_access_token_#{app_id}"
			@grant_type = 'client_credential'
			@weixin_api_url = 'https://api.weixin.qq.com'
		end

		# 获取access_token，如果启用redis则使用redis记录，如果没启用就每次进行一次查询。
    def get_access_token
    	if EasyWeixin.weixin_redis
      	access_token = weixin_redis.get(@redis_key)

      	if access_token
      		access_token
      	else
      		refresh_token
      	end

      else
      	get_access_token_http
      end
    end

    def get_access_token_http
    	conn = Faraday.new(url: "https://api.weixin.qq.com") do |faraday|
	      faraday.request :url_encoded
	      faraday.adapter :typhoeus
	    end
      response = conn.get("cgi-bin/token", authenticate_headers)

      token_hash = JSON.parse(response.body)

			access_token = token_hash["access_token"]
    end

    def refresh_token
    	get_access_token_http

			conn = Faraday.new(url: "https://api.weixin.qq.com") do |faraday|
	      faraday.request :url_encoded
	      faraday.adapter :typhoeus
	    end
      response = conn.get("cgi-bin/token", authenticate_headers)
      token_hash = JSON.parse(response.body)

			access_token = token_hash["access_token"]
			expires_in = token_hash["expires_in"]

			weixin_redis.set(@redis_key, access_token)
			weixin_redis.expire(@redis_key, expires_in - 5)

			token_hash["access_token"]
    end

		def authenticate_headers
      {grant_type: @grant_type, appid: @app_id, secret: @app_secret}
    end

    def weixin_redis
      EasyWeixin.weixin_redis
    end

    private

    	def http_post(url, data)
    		conn = Faraday.new(url: @weixin_api_url) do |faraday|
		      faraday.request :url_encoded
		      faraday.adapter :typhoeus
		    end
	      response = conn.post do |req|
	      	req.url url
	      	req.params['access_token'] = get_access_token
	      	req.headers['Content-Type'] = 'application/json'
	      	req.body = data
	      end
    	end

	end
end
